var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DrawCore } from "@bitbybit-dev/core";
import * as Inputs from "../inputs";
export class Draw extends DrawCore {
    constructor(drawHelper, context, tag) {
        super();
        this.drawHelper = drawHelper;
        this.context = context;
        this.tag = tag;
        this.defaultBasicOptions = new Inputs.Draw.DrawBasicGeometryOptions();
        this.defaultPolylineOptions = Object.assign(Object.assign({}, new Inputs.Draw.DrawBasicGeometryOptions()), { size: 2, colours: "#ff00ff" });
    }
    drawAnyAsync(inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            const entity = inputs.entity;
            if (entity === undefined || (Array.isArray(entity) && entity.length === 0)) {
                return Promise.resolve(undefined);
            }
            // we start with async ones
            if (this.detectJscadMesh(entity)) {
                return this.handleJscadMesh(inputs);
            }
            else if (this.detectOcctShape(entity)) {
                return this.handleOcctShape(inputs);
            }
            else if (this.detectOcctShapes(entity)) {
                return this.handleOcctShapes(inputs);
            }
            else if (this.detectJscadMeshes(entity)) {
                return this.handleJscadMeshes(inputs);
            }
            else {
                // here we have all sync drawer functions
                return Promise.resolve(this.drawAny(inputs));
            }
        });
    }
    /**
     * Draws any kind of geometry that does not need asynchronous computing, thus it cant be used with shapes coming from occt or jscad
     * @param inputs Contains options and entities to be drawn
     * @returns ThreeJS Group
     * @group draw sync
     * @shortname draw sync
     */
    drawAny(inputs) {
        let result;
        const entity = inputs.entity;
        if (!inputs.group) {
            if (this.detectLine(entity)) {
                result = this.handleLine(inputs);
            }
            else if (this.detectPoint(entity)) {
                result = this.handlePoint(inputs);
            }
            else if (this.detectPolyline(entity)) {
                result = this.handlePolyline(inputs);
            }
            else if (this.detectVerbCurve(entity)) {
                result = this.handleVerbCurve(inputs);
            }
            else if (this.detectVerbSurface(entity)) {
                result = this.handleVerbSurface(inputs);
            }
            else if (this.detectPolylines(entity)) {
                result = this.handlePolylines(inputs);
            }
            else if (this.detectLines(entity)) {
                result = this.handleLines(inputs);
            }
            else if (this.detectPoints(entity)) {
                result = this.handlePoints(inputs);
            }
            else if (this.detectVerbCurves(entity)) {
                result = this.handleVerbCurves(inputs);
            }
            else if (this.detectVerbSurfaces(entity)) {
                result = this.handleVerbSurfaces(inputs);
            }
            else if (this.detectTag(entity)) {
                result = this.handleTag(inputs);
            }
            else if (this.detectTags(entity)) {
                result = this.handleTags(inputs);
            }
        }
        else {
            // here types are marked on group metadata so it is not necessary to check their type
            result = this.updateAny(inputs);
        }
        return result;
    }
    handleJscadMesh(inputs) {
        return this.handleAsync(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawSolidOrPolygonMesh(Object.assign({ jscadMesh: inputs.group, mesh: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.jscadMesh);
    }
    handleJscadMeshes(inputs) {
        return this.handleAsync(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawSolidOrPolygonMeshes(Object.assign({ jscadMesh: inputs.group, meshes: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.jscadMeshes);
    }
    handleOcctShape(inputs) {
        return this.handleAsync(inputs, new Inputs.OCCT.DrawShapeDto(inputs.entity), (options) => {
            return this.drawHelper.drawShape(Object.assign(Object.assign({ shape: inputs.entity }, new Inputs.Draw.DrawOcctShapeOptions()), options));
        }, Inputs.Draw.drawingTypes.occt);
    }
    handleOcctShapes(inputs) {
        return this.handleAsync(inputs, new Inputs.OCCT.DrawShapeDto(inputs.entity), (options) => {
            return this.drawHelper.drawShapes(Object.assign(Object.assign({ shapes: inputs.entity }, new Inputs.Draw.DrawOcctShapeOptions()), options));
        }, Inputs.Draw.drawingTypes.occtShapes);
    }
    handleLine(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            const line = inputs.entity;
            return this.drawHelper.drawPolylinesWithColours(Object.assign({ polylinesMesh: inputs.group, polylines: [{ points: [line.start, line.end] }] }, options));
        }, Inputs.Draw.drawingTypes.line);
    }
    handlePoint(inputs) {
        return this.handle(inputs, this.defaultBasicOptions, (options) => {
            return this.drawHelper.drawPoint(Object.assign({ pointMesh: inputs.group, point: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.point);
    }
    handlePolyline(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawPolylineClose(Object.assign({ polylineMesh: inputs.group, polyline: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.polyline);
    }
    handleVerbCurve(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawCurve(Object.assign({ curveMesh: inputs.group, curve: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.verbCurve);
    }
    handleVerbSurface(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawSurface(Object.assign({ surfaceMesh: inputs.group, surface: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.verbSurface);
    }
    handlePolylines(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawPolylinesWithColours(Object.assign({ polylinesMesh: inputs.group, polylines: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.polylines);
    }
    handleLines(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            const lines = inputs.entity;
            return this.drawHelper.drawPolylinesWithColours(Object.assign({ polylinesMesh: inputs.group, polylines: lines.map(e => ({ points: [e.start, e.end] })) }, options));
        }, Inputs.Draw.drawingTypes.lines);
    }
    handlePoints(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawPoints(Object.assign({ pointsMesh: inputs.group, points: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.points);
    }
    handleVerbCurves(inputs) {
        return this.handle(inputs, this.defaultPolylineOptions, (options) => {
            return this.drawHelper.drawCurves(Object.assign({ curvesMesh: inputs.group, curves: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.verbCurves);
    }
    handleVerbSurfaces(inputs) {
        return this.handle(inputs, this.defaultBasicOptions, (options) => {
            return this.drawHelper.drawSurfacesMultiColour(Object.assign({ surfacesMesh: inputs.group, surfaces: inputs.entity }, options));
        }, Inputs.Draw.drawingTypes.verbSurfaces);
    }
    handleTag(inputs) {
        const options = inputs.options ? inputs.options : {
            updatable: false,
        };
        const result = this.tag.drawTag(Object.assign({ tagVariable: inputs.group, tag: inputs.entity }, options));
        result.userData = { type: Inputs.Draw.drawingTypes.tag, options };
        return result;
    }
    handleTags(inputs) {
        const options = inputs.options ? inputs.options : {
            updatable: false,
        };
        const result = this.tag.drawTags(Object.assign({ tagsVariable: inputs.group, tags: inputs.entity }, options));
        result.userData = { type: Inputs.Draw.drawingTypes.tags, options };
        return result;
    }
    updateAny(inputs) {
        let result;
        if (inputs.group && inputs.group.userData) {
            const type = inputs.group.userData.type;
            switch (type) {
                case Inputs.Draw.drawingTypes.point:
                    result = this.handlePoint(inputs);
                    break;
                case Inputs.Draw.drawingTypes.points:
                    result = this.handlePoints(inputs);
                    break;
                case Inputs.Draw.drawingTypes.line:
                    result = this.handleLine(inputs);
                    break;
                case Inputs.Draw.drawingTypes.lines:
                    result = this.handleLines(inputs);
                    break;
                case Inputs.Draw.drawingTypes.polyline:
                    result = this.handlePolyline(inputs);
                    break;
                case Inputs.Draw.drawingTypes.polylines:
                    result = this.handlePolylines(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbCurve:
                    result = this.handleVerbCurve(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbCurves:
                    result = this.handleVerbCurves(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbSurface:
                    result = this.handleVerbSurface(inputs);
                    break;
                case Inputs.Draw.drawingTypes.verbSurfaces:
                    result = this.handleVerbSurfaces(inputs);
                    break;
                case Inputs.Draw.drawingTypes.tag:
                    result = this.handleTag(inputs);
                    break;
                case Inputs.Draw.drawingTypes.tags:
                    result = this.handleTags(inputs);
                    break;
                default:
                    break;
            }
        }
        return result;
    }
    handle(inputs, defaultOptions, action, type) {
        let options = inputs.options ? inputs.options : defaultOptions;
        if (!inputs.options && inputs.group && inputs.group.userData.options) {
            options = inputs.group.userData.options;
        }
        const result = action(options);
        this.applyGlobalSettingsAndMetadataAndShadowCasting(type, options, result);
        return result;
    }
    handleAsync(inputs, defaultOptions, action, type) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = inputs.options ? inputs.options : defaultOptions;
            if (!inputs.options && inputs.group && inputs.group.userData.options) {
                options = inputs.group.userData.options;
            }
            const result = action(options);
            return result.then(r => {
                this.applyGlobalSettingsAndMetadataAndShadowCasting(type, options, r);
                return r;
            });
        });
    }
    applyGlobalSettingsAndMetadataAndShadowCasting(type, options, result) {
        const typemeta = { type, options };
        // const sgs = this.context.scene.userData.shadowGenerators as BABYLON.ShadowGenerator[];
        // result.isPickable = false;
        // result.getChildMeshes().forEach(m => { m.isPickable = false; });
        // let shadowsEnabled = true;
        // if (result.metadata && result.metadata.shadows === false) {
        //     shadowsEnabled = false;
        // }
        // if (shadowsEnabled) {
        //     if (sgs.length > 0) {
        //         result.receiveShadows = true;
        //         sgs.forEach(sg => sg.addShadowCaster(result));
        //         result.getChildMeshes().forEach(m => {
        //             m.receiveShadows = true;
        //             sgs.forEach(sg => sg.addShadowCaster(m));
        //         });
        //     }
        // }
        result.userData = result.userData ? Object.assign(Object.assign({}, result.userData), typemeta) : typemeta;
    }
}
